import functools
import time
import random
import logging
import inspect
import smawe_tools.exception as exception


class Retrying(object):

    def __init__(
            self,
            func,
            stop_max_attempt_number=None,
            wait_random_min=None,
            wait_random_max=None,
            retry_exception=None,
    ):
        args = (func, stop_max_attempt_number, wait_random_min, wait_random_max, retry_exception)
        for i, v in enumerate(args):
            if callable(v):
                if not inspect.isclass(v):
                    # 关键字传参
                    if i == 0:
                        break

                    if i == 1:
                        func, stop_max_attempt_number = stop_max_attempt_number, func
                        break
                    elif i == 2:
                        func, stop_max_attempt_number, wait_random_min = wait_random_min, func, stop_max_attempt_number
                        break
                    elif i == 3:
                        stop_max_attempt_number, wait_random_min, wait_random_max, func = \
                            func, stop_max_attempt_number, wait_random_min, wait_random_max
                        break
                    elif i == 4:
                        stop_max_attempt_number, wait_random_min, wait_random_max, retry_exception, func = args
                        break

        if not callable(func):
            raise ValueError("func param error")
        functools.update_wrapper(self, func)
        self._func = func

        self._retry_exception = retry_exception if retry_exception is not None else Exception

        self._stop_max_attempt_number = stop_max_attempt_number if stop_max_attempt_number else 1

        self._wait_random_min = wait_random_min / 1000 if isinstance(wait_random_min, int) else 0
        self._wait_random_max = wait_random_max / 1000 if isinstance(wait_random_max, int) else 1
        if self._wait_random_max <= self._wait_random_min:
            raise ValueError("wait_random_min is greater than or equal to wait_random_max")

    def __call__(self, *args, **kwargs):
        current_retry_num = 0

        while True:
            if current_retry_num > self._stop_max_attempt_number:
                raise exception.MaxRetryError("Exceeded maximum retry count error")
            try:
                if current_retry_num:
                    logging.info("\033[1;34mThis is currently the {} retry\033[0m".format(current_retry_num))
                    time.sleep(random.uniform(self._wait_random_min, self._wait_random_max))
                return self._func(*args, **kwargs)
            except self._retry_exception:
                current_retry_num += 1


def retry(*args, **kwargs):
    return functools.partial(Retrying, *args, **kwargs)



"""
字体颜色设置
    url: https://www.jianshu.com/p/a6791ba66e3d
"""
# class A:
#
#     def __init__(self):
#         self.a = 100
#
#     def __get__(self, instance, owner):
#         return object.__getattribute__(self, "a")
#
#     def __set__(self, instance, value):
#         print(instance, value)
#         self.a = value
#
#     def __del__(self):
#         del self.a
#
# class B:
#     a = A()
#
# B().a = 1
# print(B().a)

from smawe_tools.config import Config

config = Config(interpolation_level=2)
# s1 = config.switch_to_section("s1")
# s2 = config.switch_to_section("s2")
# s3 = config.switch_to_section("s3")
# s1.set("k1", "v1")
# s1.set("k2", "v2")
# s1.set("k3", "v3")
# s2.set("k3", "v3")
# s2.set("kk2", "%(k3)s-kk2")
# s2.set("kk3", "%(kk2)s-kk3")
# s2.set("kk4", "${s1:k1}")
# print(s2.get("kk2"))
# for i in s2.values():
#     print(i)
#
# for i in s1:
#     print(i)
# print(config.get_sections())
# for i in config:
#     print(i)
#
# print(len(config))
# print(len(s1))
# config.switch_to_section("s1").set("k1", "v1")
# config.switch_to_section("s1").set("k2", "v2")
# config.switch_to_section("s1").set("k3", "v3")
# print(config.switch_to_section("s1").get_options())
# config.save_config("test.ini")
# print("save successfully")
if config.read_config("test.ini"):
    print("read config successfully")
    for i in config.switch_to_section("s2").values():
        print(i)
else:
    print("read config failure")



"""
遍历Config对象时, 将返回每一个Section对象
遍历Section对象时, 将返回每一个选项的名称
"""

import configparser


class Config:
    """
    读取和保存配置设置
    """

    def __init__(self, **kwargs):
        self._default_section = kwargs.pop("default_section", configparser.DEFAULTSECT)
        i = kwargs.pop("interpolation_level", 1)
        map_set = (None, configparser.BasicInterpolation(), configparser.ExtendedInterpolation())
        try:
            self._interpolation = map_set[i]
        except TypeError:
            raise ValueError("interpolation_level param is not a integer number")
        except IndexError:
            raise ValueError("interpolation_level param must be one of (0,1,2)")
        self._config = configparser.ConfigParser(
            default_section=self._default_section, interpolation=self._interpolation, **kwargs
        )

    def set(self, section, option, value):
        """设置section中的option的值为value.
        如果配置中不存在指定的section, 则会先添加section, 然后再设置值
        """
        if not self.has_section(section):
            self._config.add_section(section)
        self._config.set(section, option, value)

    def get(self, section, option, fallback=None, **kwargs):
        """获取section中option的值, 获取不到则返回fallback"""
        return self._config.get(section, option, fallback=fallback, **kwargs)

    def save_config(self, filename):
        """保存配置"""
        with open(filename, "w", encoding="utf-8") as fp:
            self._config.write(fp)

    def read_config(self, filename):
        """
        读取配置文件设置，读取成功返回True,否则返回False
        :param filename:
        :return: bool
        """
        try:
            with open(filename, "r") as fp:
                self._config.read_file(fp)
            return True
        except (OSError, TypeError):
            return False

    def switch_to_section(self, section):
        """
        切换当前节，节如果不存在则先添加然后返回，否则切换到指定的节并返回
        :param section:
        :return: Section object
        """
        try:
            self._config.add_section(section)
        except configparser.DuplicateSectionError:
            return Section(self._config, section)
        except ValueError:
            return Section(self._config, section)
        else:
            return Section(self._config, section)

    def get_options(self, section):
        """获取所有选项"""
        return self._config.options(section)

    def get_sections(self):
        """
        获取所有可用的节, default section 不包括在该列表中
        :return: list
        """
        return self._config.sections()

    @property
    def default_section(self):
        return self._config.default_section

    @default_section.setter
    def default_section(self, value):
        self._config.default_section = value

    @property
    def boolean_states(self):
        return self._config.BOOLEAN_STATES

    @boolean_states.setter
    def boolean_states(self, value: dict):
        self._config.BOOLEAN_STATES = value

    def get_boolean(self, section, option, fallback=None, **kwargs):
        """将section中option的值转为布尔值, 可通过修改boolean_states以支持多值转换"""
        raw = kwargs.pop("raw", False)
        vars_ = kwargs.pop("vars", None)
        return self._config.getboolean(section, option, raw=raw, vars=vars_, fallback=fallback)

    def has_section(self, section):
        """配置中是否存在指定的section"""
        return self._config.has_section(section)

    def has_option(self, section, option):
        """section中是否存在指定的option"""
        return self._config.has_option(section, option)

    def defaults(self):
        """返回包含实例范围内默认值的字典。"""
        return self._config.defaults()

    def remove_section(self, section):
        """从配置中移除section"""
        return self._config.remove_section(section)

    def remove_option(self, section, option):
        """从指定的section中移除指定的option"""
        try:
            return self._config.remove_option(section, option)
        except configparser.NoSectionError:
            pass

    def __iter__(self):
        self._sections = self.get_sections()
        self._start = 0
        self._length = len(self._sections)
        return self

    def __next__(self):
        if self._start > self._length - 1:
            raise StopIteration
        self._start += 1
        return Section(self._config, self._sections[self._start - 1])

    def __len__(self):
        """返回可用的节的长度"""
        return len(self.get_sections())


class Section:

    def __init__(self, conf=None, section=None):
        self._conf: configparser.ConfigParser = conf
        self._section: configparser.SectionProxy = self._conf[section]
        self._section_name = section

    def __repr__(self):
        return "<Section: {}>".format(self._section_name)

    def get(self, option, fallback=None, **kwargs):
        """获取当前节中指定option的值"""
        return self._section.get(option, fallback=fallback, **kwargs)

    def set(self, option, value):
        """设置当前节中option的值为value"""
        self._section[option] = value

    def get_boolean(self, option, fallback=None, **kwargs):
        """将当前节中option的值转为布尔值"""
        raw = kwargs.pop("raw", False)
        vars_ = kwargs.pop("vars", None)
        return self._section.getboolean(option, raw=raw, vars=vars_, fallback=fallback)

    def items(self):
        return self._section.items()

    def keys(self):
        return self._section.keys()

    def values(self):
        return self._section.values()

    def get_options(self):
        """返回当前节的选项名列表"""
        return list(self.keys())

    def has_option(self, option):
        """当前节中是否存在指定的option"""
        return self._conf.has_option(self._section_name, option)

    def remove_option(self, option):
        """移除当前节中指定的option"""
        return self._conf.remove_option(self._section_name, option)

    def __iter__(self):
        self._iter = iter(self.keys())
        return self

    def __next__(self):
        return next(self._iter)

    def __len__(self):
        """返回可用的选项的长度"""
        return len(self.keys())


__all__ = ["Config", "Section"]



import os
import re
from os.path import join


__all__ = ['rename', 'text_conversion']


_regexs = [r"\w*?第(.*?)章.*"]
_replace_regex = r".*\第(.*?)章.*"


def rename(src=""):
    """
    对目录中的文件进行重命名
    第某某章必须在一亿章以下
    列如:
        第一百章.txt -> 第100章.txt
    :return:
    """
    src = src.strip()

    if not src:
        raise ValueError("src cannot is empty")

    root_path = src

    for f in os.listdir(src):
        if not _isfile(f):
            continue

        if not re.match(_regexs[0], f):
            continue

        if re.match(_regexs[0], f).group(1).isdigit():
            continue

        old_path = join(root_path, f)
        map_text = _calculate_lt_1yi(re.match(_regexs[0], f).group(1))
        new_path = old_path.replace(re.match(_regexs[0], f).group(1), str(map_text))
        os.rename(src=old_path, dst=new_path)


def _isfile(f: str):
    if f.__contains__(".") and f.rsplit(".", maxsplit=1)[-1]:
        return True

    return False


def _calculate_lt_w(s: str = ""):

    s = s.strip()

    if not s:
        return 0

    mapping = {
        "零": 0, "一": 1, "二": 2, "三": 3, "四": 4, "五": 5, "六": 6, "七": 7, "八": 8, "九": 9,
        "十": 10, "百": 100, "千": 1000,
        "两": 2
    }

    need = {"十", "百", "千"}

    skip = {"零"}

    # 二十， 一百零一，两千零二十
    if len(s) == 1:
        return mapping[s[0]]

    if len(s) == 2:
        # 零一
        if s[0] not in need and s[1] not in need:
            return mapping[s[0]] + mapping[s[1]]
        elif s[0] not in need and s[1] in need:
            return mapping[s[0]] * mapping[s[1]]
        else:
            raise ValueError("This conversion is not currently supported")

    result = 0
    first_num = 1
    # 九千三百零一
    for i, v in enumerate(s):

        # Encountered 0, skipping
        if v in skip:
            continue

        if i != len(s) - 1:
            if s[i] not in need and s[i+1] in need:
                first_num = mapping[v]
                continue

            if s[i] in need:
                result += first_num * mapping[v]
        else:
            if s[i] in need:
                result += first_num * mapping[v]
            else:
                result += mapping[v]

            return result


def _calculate_lt_1yi(s: str = "") -> int:

    s = s.strip()

    if not s:
        return 0

    border = "万"
    border_v = 10000

    if border in s:
        v = s.split(border)
        result = _calculate_lt_w(v[0]) * border_v + _calculate_lt_w(v[1])
        return result
    else:
        return _calculate_lt_w(s)


text_conversion = _calculate_lt_1yi



from setuptools import find_packages, setup
from smawe_tools.__version__ import version


with open("README.md", "r", encoding="utf-8") as fp:
    LONG_DESCRIPTION = fp.read()

NAME = 'smawe_tools'
DESCRIPTION = 'small tool'
EMAIL = '1281722462@qq.com'
AUTHOR = 'Samwe'
REQUIRES_PYTHON = '>=3.5.0'

about = {"__version__": version}


setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    packages=find_packages(exclude=('tests',)),
    install_requires=['colorama'],
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ]
)
